package development

import (
	"strings"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/cli"
	"shylinux.com/x/icebergs/base/ctx"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/nfs"
	"shylinux.com/x/icebergs/base/web"
	"shylinux.com/x/icebergs/core/code"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/operation/src/development/model"
)

type serve struct {
	Tables
	repos   repos
	order   string `data:"1"`
	imports string `name:"imports" role:"leader,worker"`
	autogen string `name:"autogen name* help* class* table*" icon:"bi bi-plus-square-dotted" role:"leader,worker"`
}

func (s serve) BeforeAction(m *ice.Message, arg ...string) {
	if len(arg) == 0 {
		return
	}
	kit.If(len(arg) == 1 || arg[0] != ctx.ACTION, func() { m.Option(model.REPOS_UID, arg[0]) })
	msg := m.Cmd(s.repos, s.Select, model.UID, m.Option(model.REPOS_UID), ice.Maps{mdb.TABLE: ""})
	m.Options(model.REPOS_NAME, msg.Append(model.NAME), model.REPOS_TYPE, msg.Append(model.TYPE))
}
func (s serve) Imports(m *ice.Message, arg ...string) {
	s.spaceCmdy(m, web.CODE_GIT_REPOS, "init", kit.MergeURL2(m.Option(ice.MSG_USERHOST), "/x/"+m.Option(model.REPOS_NAME)))
	s.spaceImport(m, "gonganxitong", "shylinux.com/x/community")
	s.spaceImport(m, "db/mysql", "shylinux.com/x/mysql-story")
	s.spaceCompile(m)
}
func (s serve) Autogen(m *ice.Message, arg ...string) {
	s.SpaceCmdy(m.SpawnSilent(), ice.OPS, web.CODE_COMPILE, code.DEVPACK)
	s.spaceCmdy(m, "web.team.gonganxitong.service", code.AUTOGEN, arg)
	s.spaceRepos(m, "add", "src")
	s.spaceRepos(m, "commit", "-am", "add some")
	s.spaceRepos(m, "push")
	s.spaceCompile(m)
}
func (s serve) List(m *ice.Message, arg ...string) {
	switch ReposType(kit.Int(m.Option(model.REPOS_TYPE))) {
	case ReposSystem:
		if s.spaceCmd(m, ctx.COMMAND, "web.team.gonganxitong.portal").Length() == 0 {
			s.Button(m, "", s.Imports)
			return
		}
		s.spaceCmd(m, nfs.DIR, nfs.SRC, nfs.PATH, nfs.SIZE, kit.Dict(
			nfs.DIR_DEEP, ice.TRUE, nfs.DIR_TYPE, nfs.TYPE_CAT, nfs.DIR_REG, "portal.go",
		)).Table(func(value ice.Maps) {
			m.Push(model.NAME, strings.Split(value[nfs.PATH], "/")[1])
		})
		m.PushAction(s.Open, s.Config, s.Vimer).Action(s.Autogen).Display("")
		s.Button(m, "", s.Autogen)
	}
}
func (s serve) Open(m *ice.Message, arg ...string) {
	m.ProcessOpen(s.spaceLink(m))
}
func (s serve) Config(m *ice.Message, arg ...string) {
	m.ProcessOpen(s.spaceLink(m, "view", "table"))
}
func (s serve) Vimer(m *ice.Message, arg ...string) {
	m.ProcessOpen(kit.MergeURL(web.S(m.Option(model.REPOS_NAME))+web.C("vimer"), nfs.PATH, "src/", nfs.FILE, m.Option(model.NAME)+"/portal.go"))
}

func init() { ice.TeamCtxCmd(serve{}) }

func (s serve) spaceLink(m *ice.Message, arg ...ice.Any) string {
	return kit.MergeURL(web.S(m.Option(model.REPOS_NAME))+web.C("web.team."+m.Option(model.NAME)+".portal"), arg...)
}
func (s serve) spaceRepos(m *ice.Message, arg ...string) {
	s.spaceCmdy(m, cli.SYSTEM, "git", arg)
}
func (s serve) spaceImport(m *ice.Message, arg ...string) {
	s.spaceCmdy(m, web.CODE_AUTOGEN, code.IMPORT, arg)
}
func (s serve) spaceCompile(m *ice.Message) {
	if s.spaceCmdy(m, web.CODE_VIMER, code.COMPILE); m.IsErr() {
		return
	}
	m.WaitEvent(web.OPS_SERVER_OPEN, func(msg *ice.Message, arg ...string) bool {
		return msg.Option(mdb.NAME) == m.Option(model.REPOS_NAME)
	})
}
func (s serve) spaceCmdy(m *ice.Message, arg ...ice.Any) *ice.Message {
	if m.IsErr() {
		return m
	}
	m.SetAppend().SetResult()
	return s.SpaceCmdy(m, m.Option(model.REPOS_NAME), arg...)
}
func (s serve) spaceCmd(m *ice.Message, arg ...ice.Any) *ice.Message {
	if m.IsErr() {
		return m
	}
	m.SetAppend().SetResult()
	return s.SpaceCmd(m, m.Option(model.REPOS_NAME), arg...)
}
